package lcsb.mapviewer.api.projects;

import java.io.Serializable;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import lcsb.mapviewer.services.view.OverviewImageView;

public class ProjectMetaData implements Serializable {

  /**
   * Default class logger.
   */
  @SuppressWarnings("unused")
  private static Logger logger = Logger.getLogger(ProjectMetaData.class);

  /**
   * 
   */
  private static final long serialVersionUID = 1L;

  /**
   * Version of the project.
   */
  private String version;

  private Map<String, Object> disease;

  private Map<String, Object> organism;

  private Integer idObject;
  private String status;
  private Double progress;
  private String notifyEmail;
  private Boolean warnings;
  private Boolean errors;

  /**
   * Name of the project.
   */
  private String name;

  /**
   * {@link lcsb.mapviewer.model.Project#projectId Project identifier}.
   */
  private String projectId;

  /**
   * List of overview images attached to this model.
   */
  private List<OverviewImageView> overviewImageViews;

  /**
   * Top level overview image.
   */
  private OverviewImageView topOverviewImage;

  /**
   * Default constructor. Should be used only for deserialization.
   */
  protected ProjectMetaData() {
  }

  /**
   * @return the version
   * @see #version
   */
  public String getVersion() {
    return version;
  }

  /**
   * @param version
   *          the version to set
   * @see #version
   */
  public void setVersion(String version) {
    this.version = version;
  }

  /**
   * @return the name
   * @see #name
   */
  public String getName() {
    return name;
  }

  /**
   * @param name
   *          the name to set
   * @see #name
   */
  public void setName(String name) {
    this.name = name;
  }

  /**
   * @return the overviewImageViews
   * @see #overviewImageViews
   */
  public List<OverviewImageView> getOverviewImageViews() {
    return overviewImageViews;
  }

  /**
   * @param overviewImageViews
   *          the overviewImageViews to set
   * @see #overviewImageViews
   */
  public void setOverviewImageViews(List<OverviewImageView> overviewImageViews) {
    this.overviewImageViews = overviewImageViews;
  }

  /**
   * @return the topOverviewImage
   * @see #topOverviewImage
   */
  public OverviewImageView getTopOverviewImage() {
    return topOverviewImage;
  }

  /**
   * @param topOverviewImage
   *          the topOverviewImage to set
   * @see #topOverviewImage
   */
  public void setTopOverviewImage(OverviewImageView topOverviewImage) {
    this.topOverviewImage = topOverviewImage;
  }

  /**
   * @return the projectId
   * @see #projectId
   */
  public String getProjectId() {
    return projectId;
  }

  /**
   * @param projectId
   *          the projectName to set
   * @see #projectId
   */
  public void setProjectId(String projectId) {
    this.projectId = projectId;
  }

  /**
   * @return the idObject
   * @see #idObject
   */
  public Integer getIdObject() {
    return idObject;
  }

  /**
   * @param idObject
   *          the idObject to set
   * @see #idObject
   */
  public void setIdObject(Integer idObject) {
    this.idObject = idObject;
  }

  /**
   * @return the disease
   * @see #disease
   */
  public Map<String, Object> getDisease() {
    return disease;
  }

  /**
   * @param disease
   *          the disease to set
   * @see #disease
   */
  public void setDisease(Map<String, Object> disease) {
    this.disease = disease;
  }

  /**
   * @return the organism
   * @see #organism
   */
  public Map<String, Object> getOrganism() {
    return organism;
  }

  /**
   * @param organism
   *          the organism to set
   * @see #organism
   */
  public void setOrganism(Map<String, Object> organism) {
    this.organism = organism;
  }

  /**
   * @return the status
   * @see #status
   */
  public String getStatus() {
    return status;
  }

  /**
   * @param status
   *          the status to set
   * @see #status
   */
  public void setStatus(String status) {
    this.status = status;
  }

  /**
   * @return the progress
   * @see #progress
   */
  public Double getProgress() {
    return progress;
  }

  /**
   * @param progress
   *          the progress to set
   * @see #progress
   */
  public void setProgress(Double progress) {
    this.progress = progress;
  }

  /**
   * @return the notifyEmail
   * @see #notifyEmail
   */
  public String getNotifyEmail() {
    return notifyEmail;
  }

  /**
   * @param notifyEmail
   *          the notifyEmail to set
   * @see #notifyEmail
   */
  public void setNotifyEmail(String notifyEmail) {
    this.notifyEmail = notifyEmail;
  }

  public Boolean getWarnings() {
    return warnings;
  }

  public void setWarnings(Boolean warnings) {
    this.warnings = warnings;
  }

  public Boolean getErrors() {
    return errors;
  }

  public void setErrors(Boolean errors) {
    this.errors = errors;
  }
}
