package lcsb.mapviewer.api.projects.models.functions;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.transform.TransformerException;

import org.apache.log4j.Logger;
import org.springframework.transaction.annotation.Transactional;

import lcsb.mapviewer.api.BaseRestImpl;
import lcsb.mapviewer.api.ObjectNotFoundException;
import lcsb.mapviewer.api.QueryException;
import lcsb.mapviewer.common.exception.InvalidXmlSchemaException;
import lcsb.mapviewer.model.map.kinetics.SbmlFunction;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.services.SecurityException;

@Transactional(value = "txManager")
public class FunctionsRestImpl extends BaseRestImpl {

  /**
   * Default class logger.
   */
  @SuppressWarnings("unused")
  private Logger logger = Logger.getLogger(FunctionsRestImpl.class);

  public Map<String, Object> getFunction(String projectId, String modelId, String token, String functionId)
      throws SecurityException, QueryException {
    List<Model> models = getModels(projectId, modelId, token);
    int id = Integer.valueOf(functionId);
    for (Model model : models) {
      for (SbmlFunction function : model.getFunctions()) {
        if (function.getId() == id) {
          return functionToMap(function);
        }
      }
    }
    throw new ObjectNotFoundException("Function with given id doesn't exist");
  }

  private Map<String, Object> functionToMap(SbmlFunction function) {
    Map<String, Object> result = new HashMap<>();
    result.put("id", function.getId());
    result.put("functionId", function.getFunctionId());
    result.put("name", function.getName());
    result.put("definition", function.getDefinition());
    try {
      result.put("mathMlPresentation", super.mathMLToPresentationML(function.getDefinition()));
    } catch (IOException | InvalidXmlSchemaException | TransformerException e) {
      logger.error("Problems with transforming kinetics", e);
    }
    result.put("arguments", function.getArguments());
    return result;
  }

  public List<Map<String, Object>> getFunctions(String projectId, String modelId, String token) throws SecurityException {
    List<Map<String, Object>> result = new ArrayList<>();
    List<Model> models = getModels(projectId, modelId, token);
    for (Model model : models) {
      for (SbmlFunction function : model.getFunctions()) {
        result.add(functionToMap(function));
      }
    }
    return result;
  }
}
