package lcsb.mapviewer.api.projects.models;

import static org.junit.Assert.assertEquals;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyString;
import static org.mockito.Matchers.refEq;

import java.util.List;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;

import lcsb.mapviewer.api.RestTestFunctions;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.services.interfaces.IModelService;

public class ModelRestImplTest extends RestTestFunctions {

  @Autowired
  ModelRestImpl _projectRestImpl;

  @AfterClass
  public static void tearDownAfterClass() throws Exception {
  }

  @Before
  public void setUp() throws Exception {
  }

  @After
  public void tearDown() throws Exception {
  }

  @Test
  public void testModels() throws Exception {
    try {
      ModelRestImpl modelRest = createMockProjectRest("testFiles/model/sample.xml");
      List<ModelMetaData> result = modelRest.getModels("sample", token.getId());
      assertEquals(1, result.size());
    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    }
  }
  
  @Test
  public void testGetEmptyModels() throws Exception {
    try {
      ModelRestImpl modelRest = createEmptyProjectRest("test");
      List<ModelMetaData> result = modelRest.getModels("test", token.getId());
      assertEquals(0, result.size());
    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    }
  }

  private ModelRestImpl createEmptyProjectRest(String string) {
    IModelService mockModelService = Mockito.mock(IModelService.class);
    Mockito.when(mockModelService.getLastModelByProjectId(refEq(string), any())).thenReturn(null);
    _projectRestImpl.setModelService(mockModelService);
    return _projectRestImpl;
  }

  private ModelRestImpl createMockProjectRest(String string) throws Exception {
    Model model = super.getModelForFile(string, true);
    IModelService mockModelService = Mockito.mock(IModelService.class);
    Mockito.when(mockModelService.getLastModelByProjectId(anyString(), any())).thenReturn(model);
    _projectRestImpl.setModelService(mockModelService);
    return _projectRestImpl;
  }

}
